/*
 *  ngtk/ngtkdialog/ngtkdialog.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *	The main function was taken and slightly modified from
 *	'Lxdialog' by William Roadcap <roadcap@cfw.com>, itself derived from
 *	'dialog', by Savio Lam <lam836@cs.cuhk.hk>
 *
 *	Don't bother anyone if you experience problems with this program, but me
 *	(Gabriel Fort)
 */

/*----------------------------------------------------------------------------
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>
#include "ngtkdialog.h"

static void Usage(const char *name);

typedef int (jumperFn) (const char *title, int argc, const char * const * argv);

struct Mode {
    char *name;
    int argmin, argmax, argmod;
    jumperFn *jumper;
};

jumperFn j_menu, j_checklist, j_radiolist, j_yesno, j_textbox, j_inputbox;
jumperFn j_msgbox, j_infobox;

static struct Mode modes[] =
{
    {"--menu", 9, 0, 3, j_menu},
    {"--checklist", 9, 0, 3, j_checklist},
    {"--radiolist", 9, 0, 3, j_radiolist},
    {"--yesno",    5,5,1, j_yesno},
    {"--textbox",  5,5,1, j_textbox},
    {"--inputbox", 5, 6, 1, j_inputbox},
    {"--msgbox", 5, 5, 1, j_msgbox},
    {"--infobox", 5, 5, 1, j_infobox},
    {NULL, 0, 0, 0, NULL}
};

static struct Mode *modePtr;
const char *backtitle = NULL;
const char *title = NULL;

int main(int argc, const char * const * argv)
{
	int offset = 0, clear_screen = 0, end_common_opts = 0, retval;
    ngtk_win_t *wall;

	if (argc < 2) {
		Usage (argv[0]);
		exit (-1);
	}

	while (offset < argc - 1 && !end_common_opts) {	/* Common options */
		if (!strcmp (argv[offset + 1], "--title")) {
			if (argc - offset < 3 || title != NULL) {
				Usage (argv[0]);
				exit (-1);
			} else {
				title = argv[offset + 2];
				offset += 2;
			}
		} else
			if (!strcmp (argv[offset + 1], "--backtitle")) {
				if (backtitle != NULL) {
					Usage (argv[0]);
					exit (-1);
				} else {
					backtitle = argv[offset + 2];
					offset += 2;
				}
			} else
				if (!strcmp (argv[offset + 1], "--clear")) {
					if (clear_screen) {	/* Hey, "--clear" can't appear twice! */
						Usage (argv[0]);
						exit (-1);
					} else
						if (argc == 2) {	/* we only want to clear the screen */
							ngtk_launch_display();
							wall = ngtk_push_win(LINES, COLS, 0, 0, YELLOW_F(BLUE_B), A_BOLD, 0, 0);
							ngtk_clrwin(wall->win, 32, 1);
							ngtk_stop_display();
							return 0;
						} else {
							clear_screen = 1;
							offset++;
						}
				} else			/* no more common options */
					end_common_opts = 1;
	}

	if (argc - 1 == offset) {	/* no more options */
		Usage (argv[0]);
		exit (-1);
	}
    /* use a table to look for the requested mode, to avoid code duplication */

	for (modePtr = modes; modePtr->name; modePtr++)	/* look for the mode */
		if (!strcmp (argv[offset + 1], modePtr->name))
			break;

	if (!modePtr->name)
		Usage (argv[0]);
	if (argc - offset < modePtr->argmin)
		Usage (argv[0]);
	if (modePtr->argmax && argc - offset > modePtr->argmax)
		Usage (argv[0]);

	ngtk_launch_display();
	retval = (*(modePtr->jumper))(title, argc - offset, argv + offset);

	ngtk_fade_out();
	ngtk_stop_display();
	ngtk_fade_in();

	exit(retval);
}

/*
 * Print program usage
 */
static void Usage(const char *name)
{
    fprintf (stderr, "\
\nNgtkDialog, by Gabriel Fort (gabx@tfz.net)\
\nA dialog/lxdialog clone, based on the ngtk library.\
\n\
\n Display dialog boxes from shell scripts.\
\n\
\nUsage: %s --clear\
\n       %s [--title <title>] [--backtitle <backtitle>] --clear <Box options>\
\n\
\nBox options:\
\n\
\n  --menu      <text> <height> <width> <menu height> <tag1> <item1>...\
\n  --checklist <text> <height> <width> <list height> <tag1> <item1> <status1>...\
\n  --radiolist <text> <height> <width> <list height> <tag1> <item1> <status1>...\
\n  --textbox   <file> <height> <width>\
\n  --inputbox  <text> <height> <width> [<init>]\
\n  --yesno     <text> <height> <width>\
\n", name, name);
    exit (-1);
}

/*
 * These are the program jumpers
 */

int j_menu(const char *t, int ac, const char * const * av)
{
/*    return dialog_menu (t, av[2], atoi (av[3]), atoi (av[4]),
			atoi (av[5]), av[6], (ac - 6) / 2, av + 7);*/
	return def_err();
}

int j_checklist(const char *t, int ac, const char * const * av)
{
/*    return dialog_checklist (t, av[2], atoi (av[3]), atoi (av[4]),
	atoi (av[5]), (ac - 6) / 3, av + 6, FLAG_CHECK);*/
	return def_err();
}

int j_radiolist(const char *t, int ac, const char * const * av)
{
/*    return dialog_checklist (t, av[2], atoi (av[3]), atoi (av[4]),
	atoi (av[5]), (ac - 6) / 3, av + 6, FLAG_RADIO);*/
	return def_err();
}

int j_textbox(const char *t, int ac, const char * const * av)
{
//    return dialog_textbox (t, av[2], atoi (av[3]), atoi (av[4]));
	return def_err();
}

int j_yesno(const char *t, int ac, const char * const * av)
{
//    return dialog_yesno (t, av[2], atoi (av[3]), atoi (av[4]));
	return(yesno(atoi(av[3]), atoi(av[4]), av[2]));
}

int j_inputbox(const char *t, int ac, const char * const * av)
{
/*    int ret = dialog_inputbox (t, av[2], atoi (av[3]), atoi (av[4]),
                            ac == 6 ? av[5] : (char *) NULL);
    if (ret == 0)
        fprintf(stderr, dialog_input_result);
    return ret;*/

	int ret = inputbox(atoi(av[3]), atoi(av[4]), av[2], ac == 6 ? av[5]:(char *)NULL);

	return ret;
}

int j_msgbox(const char *t, int ac, const char * const * av)
{
//    return dialog_msgbox (t, av[2], atoi (av[3]), atoi (av[4]), 1);
	return def_err();
}

int j_infobox(const char *t, int ac, const char * const * av)
{
//    return dialog_msgbox (t, av[2], atoi (av[3]), atoi (av[4]), 0);
	return def_err();
}

