/*
 *  ngtk/c/ngtk_menu.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 */

/*------------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>

ngtk_menu_item *ngtk_mk_menu_items(const int n_its, const char *texts[n_its-1],
                                   const void *funcs[n_its-1])
{
    int i;
    int fill_size = sizeof(ngtk_menu_item);
    ngtk_menu_item *fill = (ngtk_menu_item *)malloc(fill_size);

    for (i=0;i<=n_its-1;i++) {
        fill_size += strlen(texts[i])+sizeof(void *);
        fill = (ngtk_menu_item *)realloc(fill, fill_size);
        fill[i].text = (char *)malloc(strlen(texts[i])+1);
        strcpy(fill[i].text, texts[i]);
        fill[i].func = funcs[i];
    }

    return(fill);
}

ngtk_menu_t *ngtk_mk_menu(const char *title, const int item_count,
						  const char *texts[item_count-1],
						  const void *funcs[item_count-1])
{
    int i;
    int buf = 0;
    int max_len = 0;
    ngtk_menu_t *ret_menu = (ngtk_menu_t *)malloc(256);

    ret_menu->title = (char *)malloc(strlen(title));
    strcpy(ret_menu->title, title);

    ret_menu->items = ngtk_mk_menu_items(item_count, texts, funcs);
    ret_menu->item_count = item_count;

    for (i=0;i<item_count;i++) {
        buf = strlen(texts[i]);
        if (buf>max_len)
            max_len = buf;
    }

    ret_menu->max_item_len = max_len;
    ret_menu->current_item = 0;
    ret_menu->previous_item = 1;
    ret_menu->visible = 0;
	ret_menu->clear_flag = 0;
	ret_menu->boxed = 0;

    return(ret_menu);
}

ngtk_menu_t *ngtk_show_menu(ngtk_menu_t *c_menu, int top, int left, int bcpair, int battr,
	                        int fcpair, int fattr, int box)
{
    int i;
	int offs = 0;

    if (c_menu->visible == 1)
        return(c_menu);

    c_menu->top = top;
    c_menu->left = left;
    c_menu->fcpair = fcpair;
    c_menu->bcpair = bcpair;
    c_menu->fattr = fattr;
    c_menu->battr = battr;

	if (box) {
		offs = 1;
		c_menu->boxed = 1;
		c_menu->box = ngtk_mkbox(c_menu->item_count+2, c_menu->max_item_len+4, \
								 top, left, bcpair, battr, c_menu->title, 0, 1, 1);
	}

    c_menu->container = ngtk_push_win(c_menu->item_count, c_menu->max_item_len+2, \
								   top+offs, left+offs, bcpair, battr, 0, 0);

	ngtk_clrwin(c_menu->container->win, 32, 1);

    for (i=0;i<c_menu->item_count;i++) {
        mvwprintw(c_menu->container->win, i, 1, c_menu->items[i].text);
    }

    c_menu->visible = 1;
    return(c_menu);
}


ngtk_menu_t *ngtk_hide_menu(ngtk_menu_t *c_menu)
{
    if (!c_menu->visible)
        return(c_menu);

    ngtk_pull_win(c_menu->container);
    ngtk_pop_win(c_menu->container);

	if (c_menu->boxed)
		ngtk_pop_win(c_menu->box);

    c_menu->visible = 0;

    return(c_menu);
}

void ngtk_move_selection(ngtk_menu_t *c_menu)
{
    char *tab = (char *)malloc(80);
    int tablen;
	WINDOW *win = c_menu->container->win;

    tablen = (c_menu->max_item_len+1)-strlen(c_menu->items[c_menu->current_item].text);
    memset(tab, 32, tablen);
    tab[tablen] = 0;
    wattrset(win, COLOR_PAIR(c_menu->fcpair) | c_menu->fattr);
    mvwprintw(win, c_menu->current_item, 0, " %s%s", c_menu->items[c_menu->current_item].text, tab);

    tablen = (c_menu->max_item_len+1)-strlen(c_menu->items[c_menu->previous_item].text);
    memset(tab, 32, tablen);
    tab[tablen] = 0;
    wattrset(win, COLOR_PAIR(c_menu->bcpair) | c_menu->battr);
    mvwprintw(win, c_menu->previous_item, 0, " %s%s", c_menu->items[c_menu->previous_item].text, tab);

    free(tab);
}

void *ngtk_manage_menu(ngtk_menu_t *c_menu)
{
    int buf = 0;
	WINDOW *win = c_menu->container->win;

    ngtk_move_selection(c_menu);

    wrefresh(win);

    while (buf != CR) {
        buf = ngtk_get_keysym();

        switch(buf) {
            case NGTK_KEY_UP:
                if (c_menu->current_item == 0) {
                    c_menu->previous_item=0;
                    c_menu->current_item = c_menu->item_count-1;
                } else {
                    c_menu->previous_item = c_menu->current_item;
                    c_menu->current_item--;
                }
                break;

            case NGTK_KEY_DOWN:
                if (c_menu->current_item == c_menu->item_count-1) {
                    c_menu->previous_item = c_menu->item_count-1;
                    c_menu->current_item = 0;
                } else {
                    c_menu->previous_item = c_menu->current_item;
                    c_menu->current_item++;
                }
				break;

			case ESC:
				c_menu->clear_flag = 1;
				return(&ngtk_dumb_func);
				break;

            default:
                break;
        }
        ngtk_move_selection(c_menu);
        wrefresh(win);
    }
	return(c_menu->items[c_menu->current_item].func);
/*    return(c_menu);*/
}

void ngtk_destroy_menu(ngtk_menu_t *c_menu)
{
    int i;

	if (c_menu->visible)
		ngtk_hide_menu(c_menu);

    for (i=0;i<=c_menu->item_count-1;i++)
        free(c_menu->items[i].text);

    free(c_menu->title);
    free(c_menu);
}
